(* Convert svg image to transparent png

    Dr. J. Rathlev, D-24222 Schwentinental

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   J. Rathlev, September 2024
   last modified: September 2024
   *)

unit SvgToPngMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, Vcl.Shell.ShellCtrls,
  Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

const
  Vers = '1.0.0';
  CopRgt = ' 2024 Dr. J. Rathlev, D-24222 Schwentinental';
  EmailAdr = 'kontakt(a)rathlev-home.de';

  MaxSizes = 4;

type
  TMainForm = class(TForm)
    Label1: TLabel;
    btnImgDir: TSpeedButton;
    edtImgDir: TComboBox;
    bbConvert: TBitBtn;
    ShellListView: TShellListView;
    ShellComboBox: TShellComboBox;
    bbInfo: TBitBtn;
    bbExit: TBitBtn;
    StatusBar: TStatusBar;
    btSelectAll: TBitBtn;
    btSelectNone: TBitBtn;
    edtPngDir: TComboBox;
    Label2: TLabel;
    btnPngDir: TSpeedButton;
    paStatus: TPanel;
    paMain: TPanel;
    meStatus: TMemo;
    spStatus: TSplitter;
    gbSizes: TGroupBox;
    cb032: TCheckBox;
    cb064: TCheckBox;
    cb128: TCheckBox;
    cb256: TCheckBox;
    OpenDialog: TOpenDialog;
    cbIcon: TCheckBox;
    cb048: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ShellListViewAddFolder(Sender: TObject; AFolder: TShellFolder;
      var CanAdd: Boolean);
    procedure bbInfoClick(Sender: TObject);
    procedure btnImgDirClick(Sender: TObject);
    procedure btSelectAllClick(Sender: TObject);
    procedure btSelectNoneClick(Sender: TObject);
    procedure bbExitClick(Sender: TObject);
    procedure bbConvertClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure edtImgDirCloseUp(Sender: TObject);
    procedure btnPngDirClick(Sender: TObject);
    procedure edtPngDirCloseUp(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ShellListViewClick(Sender: TObject);
  private
    { Private-Deklarationen }
    AppPath,IniName,
    LastPath,LastDest,
    DefPath,FileMask,
    InkScapeProg : string;
    PngSizes : array [0..MaxSizes] of TCheckBox;
    procedure ShowStatus;
    procedure UpdateDirList;
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses System.Masks, System.IniFiles,
  GnuGetText, InitProg, WinShell, ShellDirDlg, PathUtils, ListUtils, WinUtils,
  MsgDialogs, StringUtils, WinExecute, GraphUtils;

const
  PngExt = 'png';
  PngSubDir = PngExt;
  IcoExt = 'ico';
  SvgMask = '*.svg';

  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  ImgDirSekt = 'ImageDirs';
  DestSekt   = 'PngDirs';

  iniTop      = 'Top';
  iniLeft     = 'Left';
  iniInkScape = 'InkScape';
  iniLast     = 'LastDir';
  iniDest     = 'LastDest';
  iniSizes    = 'PngSizes';
  iniIcon     = 'CreateIcon';

  InkScapePath = 'InkScape\bin\';
  InkScapeExe = 'inkscape.exe';

  ExpOpt = ' --export-filename ';
  ScaleOpt = ' -C -w %0:u -h %0:u -y 0';  // page area, width, height, transparent background

  BitMask : array [0..MaxSizes] of word = (1,2,4,8,16);

{ ------------------------------------------------------------------- }
procedure TMainForm.FormCreate(Sender: TObject);
var
  IniFile : TMemIniFile;
  n,i     : integer;
begin
  TranslateComponent(self);
  InitPaths(AppPath,DefPath);
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  IniFile:=TMemIniFile.Create(IniName);
  with IniFile do begin
    Top:=ReadInteger(CfgSekt,iniTop,Top);
    Left:=ReadInteger(CfgSekt,iniLeft,Left);
    InkScapeProg:=ReadString(CfgSekt,iniInkScape,'');
    LastPath:=ReadString(CfgSekt,IniLast,DefPath);
    LastDest:=ReadString(CfgSekt,IniDest,'png');
    n:=ReadInteger(CfgSekt,iniSizes,15);
    cbIcon.Checked:=ReadBool(CfgSekt,iniIcon,true);
    end;
  LoadHistory(IniFile,ImgDirSekt,edtImgDir);
  LoadHistory(IniFile,DestSekt,edtPngDir);
  IniFile.Free;
  FileMask:=SvgMask;
//  InkScapePath:='c:\Program Files\Inkscape\bin\inkscape.exe';
  PngSizes[0]:=cb032; PngSizes[1]:=cb048; PngSizes[2]:=cb064; PngSizes[3]:=cb128; PngSizes[4]:=cb256;
  for i:=0 to MaxSizes do PngSizes[i].Checked:=n and BitMask[i]<>0;
  end;

procedure TMainForm.FormDestroy(Sender: TObject);
var
  IniFile : TMemIniFile;
  n,i : integer;
begin
  IniFile:=TMemIniFile.Create(IniName);
  with IniFile do begin
    WriteInteger(CfgSekt,iniTop,Top);
    WriteInteger(CfgSekt,iniLeft,Left);
    WriteString(CfgSekt,iniInkScape,InkScapeProg);
    WriteString(CfgSekt,IniLast,edtImgDir.Text);
    WriteString(CfgSekt,IniDest,edtPngDir.Text);
    n:=0;
    for i:=0 to MaxSizes do if PngSizes[i].Checked then n:=n or BitMask[i];
    WriteInteger(CfgSekt,iniSizes,n);
    WriteBool(CfgSekt,iniIcon,cbIcon.Checked);
    end;
  SaveHistory(IniFile,ImgDirSekt,true,edtImgDir);
  SaveHistory(IniFile,DestSekt,true,edtPngDir);
  with IniFile do begin
    UpdateFile; Free;
    end;
  end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  with ShellListView do SetColWidths([GetWidth-46,12,12,18]);
  end;

procedure TMainForm.FormShow(Sender: TObject);
var
  ok : boolean;
begin
  if InkScapeProg.IsEmpty or not FileExists(InkScapeProg) then begin
    // check for 64-bit installation
    InkScapeProg:=IncludeTrailingPathDelimiter(GetProgramFolder(pfProgramFiles64))+InkScapePath+InkScapeExe;
    ok:=FileExists(InkScapeProg);
    if not ok then begin  // check for 32-bit installation
      InkScapeProg:=IncludeTrailingPathDelimiter(GetProgramFolder(pfProgramFiles64))+InkScapePath+InkScapeExe;
      ok:=FileExists(InkScapeProg);
      end;
    if not ok then begin
      with OpenDialog do begin
        InitialDir:=GetProgramFolder(pfProgramFiles64);
        Filename:='';
        Filter:=_('Programs|*.exe'); //'Programme|*.exe'
        Title:=_('Seek for InkScape');
        if Execute then InkScapeProg:=Filename
        else begin
          ErrorDialog(Caption,_('InkScape was not found on this computer!'+sLineBreak
            +'Converting of SVG files is not possible!'));
          Close;
          end;
        end;
      end;
    end;
  AddToHistory(edtImgDir,LastPath);
  AddToHistory(edtPngDir,LastDest);
  UpdateDirList;
  with ShellListView do if Items.Count>0 then Items[0].Selected:=true;
  ShowStatus;
  end;

procedure TMainForm.ShowStatus;
begin
  with ShellListView do if Items.Count>0 then begin
    if SelCount>0 then Statusbar.SimpleText:=GetPluralString(_('image of'),_('images of'),SelCount)+
      Format(_(' %u selected'),[Items.Count])
    else Statusbar.SimpleText:=GetPluralString(_('matching image'),_('matching images'),Items.Count);
    end
  else Statusbar.SimpleText:='';
  end;

procedure TMainForm.ShellListViewAddFolder(Sender: TObject;
  AFolder: TShellFolder; var CanAdd: Boolean);
begin
  CanAdd:=MatchesMask(AFolder.DisplayName,FileMask);
  end;

procedure TMainForm.ShellListViewClick(Sender: TObject);
begin
  ShowStatus;
  end;

procedure TMainForm.bbExitClick(Sender: TObject);
begin
  Close;
  end;

procedure TMainForm.bbInfoClick(Sender: TObject);
begin
  InfoDialog(TopLeftPos(bbExit),Caption+' - '+GetVersion(3,Vers)
    +' ('+DateToStr(FileDateToDateTime(FileAge(Application.ExeName)))+')'+sLineBreak
    +CopRgt+sLineBreak+'E-Mail: '+EMailAdr);
  end;

procedure TMainForm.btnImgDirClick(Sender: TObject);
var
  s : string;
begin
  s:=GetExistingParentPath(edtImgDir.Text,DefPath);
  if ShellDirDialog.Execute (_('Directory with SVG images'),true,true,false,'',s) then begin
    AddToHistory(edtImgDir,s);
    UpdateDirList;
    ShowStatus;
    end;
  end;

procedure TMainForm.btnPngDirClick(Sender: TObject);
var
  s : string;
begin
  s:=edtPngDir.Text;
  if not ContainsFullPath(s) then s:=AddPath(edtImgDir.Text,s);
  s:=GetExistingParentPath(s,DefPath);
  if ShellDirDialog.Execute (_('Directory for PNG images'),true,true,false,'',s) then begin
    s:=MakeRelativePath(edtImgDir.Text,s);
    AddToHistory(edtPngDir,s);
    end;
  end;

procedure TMainForm.UpdateDirList;
begin
  with edtImgDir do ShellComboBox.Path:=Items[ItemIndex];
  end;

procedure TMainForm.edtImgDirCloseUp(Sender: TObject);
begin
  UpdateHistory(edtImgDir);
  UpdateDirList;
  ShowStatus;
  end;

procedure TMainForm.edtPngDirCloseUp(Sender: TObject);
begin
  UpdateHistory(edtPngDir);
  end;

procedure TMainForm.btSelectNoneClick(Sender: TObject);
var
  i : integer;
begin
  with ShellListView do for i:=0 to Items.Count-1 do Items[i].Selected:=false;
  ShowStatus;
  end;

procedure TMainForm.btSelectAllClick(Sender: TObject);
var
  i : integer;
begin
  with ShellListView do for i:=0 to Items.Count-1 do Items[i].Selected:=true;
  ShowStatus;
  end;

procedure TMainForm.bbConvertClick(Sender: TObject);
var
  sd,sn,sp,s : string;
  hr         : HResult;
  sl,pl      : TStringList;
  n,i,j,m,k  : integer;
  err        : boolean;
begin
  Cursor:=crHourglass;
  sd:=edtImgDir.Text; sp:=edtPngDir.Text;
  if not ContainsFullPath(sp) then sp:=AddPath(sd,sp);
  ForceDirectories(sp);
  sl:=TStringList.Create; pl:=TStringList.Create; meStatus.Clear;
  n:=0;
  with ShellListView do for i:=0 to Items.Count-1 do if Items[i].Selected then begin
    sn:=Folders[i].DisplayName;
    err:=false; pl.Clear;
    for m:=0 to MaxSizes do with PngSizes[m] do if Checked then begin
      s:=AddNameSuffix(sn,Format('-%.3u',[Tag]),PngExt);
      k:=meStatus.Lines.Add(_('Converting')+ColSpace+sn+' -> '+AddPath(edtPngDir.Text,s));
      s:=AddPath(sp,s); pl.Add(s);
      s:=MakeQuotedStr(InkScapeProg)+Space+MakeQuotedStr(AddPath(sd,sn))+ExpOpt+MakeQuotedStr(s)+Format(ScaleOpt,[Tag]);
      hr:=ExecuteConsoleProcess(s,'',sl,cpUtf8);
      if hr<>0 then begin
        meStatus.Lines.Add('  *** '+_('Error')+ColSpace+SysErrorMessage(hr));
        meStatus.Lines.Add('  *** '+_('Command')+ColSpace+s);
        err:=true;
        end
      else begin
        if sl.Count>0 then begin
          for j:=0 to sl.Count-1 do if length(Trim(sl[j]))>0 then meStatus.Lines.Add('  * '+sl[j]);
          err:=true;
          end
        else begin
          with meStatus do Lines[k]:=Lines[k]+' -> '+_('Done');
          inc(n);
          end;
        end;
      if err then Break;
      end;
    if n>0 then begin
      if not err then begin
        meStatus.Lines.Add(' => '+Format(_('%u png images were created'),[n]));
        if cbIcon.Checked then begin
          sn:=AddPath(sp,NewExt(sn,IcoExt));
          if CreateIconFromPng(sn,pl) then
            meStatus.Lines.Add(' => '+Format(_('Icon "%s" was created'),[sn]))
          else meStatus.Lines.Add('  *** '+Format(_('Error writing to "%s"'),[sp]));
          end;
        end
      end
    else meStatus.Lines.Add('  *** '+_('No image sizes selected'));
    end;
  meStatus.Lines.Add(' => '+_('Done'));
  Cursor:=crDefault;
  sl.Free; pl.Free;
  end;

end.
